<?php

declare(strict_types=1);

namespace Dedi\SyliusPayboxPlugin\Payum\Action;

use Dedi\SyliusPayboxPlugin\Payum\PayboxApi;
use Payum\Core\Action\ActionInterface;
use Payum\Core\ApiAwareInterface;
use Payum\Core\Exception\RequestNotSupportedException;
use Payum\Core\Exception\UnsupportedApiException;
use Payum\Core\Request\Capture;
use Sylius\Component\Core\Model\PaymentInterface;
use Symfony\Component\HttpFoundation\RequestStack;

final class CaptureAction implements ActionInterface, ApiAwareInterface
{
    private PayboxApi $api;

    private RequestStack $requestStack;

    public function __construct(RequestStack $requestStack)
    {
        $this->requestStack = $requestStack;
    }

    public function execute($request)
    {
        RequestNotSupportedException::assertSupports($this, $request);

        $status = $this->requestStack->getCurrentRequest()->get('status', null);

        /** @var PaymentInterface $payment */
        $payment = $request->getModel();

        if (null === $status) {
            $this->api->doPayment($payment->getDetails(), $payment->getOrder(), $request->getToken());
        } else {
            $payment->setDetails(['status' => $status]);
        }
    }

    public function supports($request)
    {
        return
            $request instanceof Capture &&
            $request->getModel() instanceof PaymentInterface
        ;
    }

    public function setApi($api)
    {
        if (!$api instanceof PayboxApi) {
            throw new UnsupportedApiException('Not supported. Expected an instance of ' . PayboxApi::class);
        }

        $this->api = $api;
    }
}
