<?php

declare(strict_types=1);

namespace Dedi\SyliusPayboxPlugin\Payum\Action;

use Dedi\SyliusPayboxPlugin\Payum\PayboxApi;
use Payum\Core\Action\ActionInterface;
use Payum\Core\Exception\RequestNotSupportedException;
use Payum\Core\Request\GetStatusInterface;
use Sylius\Component\Core\Model\PaymentInterface;

final class StatusAction implements ActionInterface
{
    /**
     * @param GetStatusInterface $request
     */
    public function execute($request)
    {
        RequestNotSupportedException::assertSupports($this, $request);

        /** @var PaymentInterface $payment */
        $payment = $request->getFirstModel();
        $details = $payment->getDetails();

        if (null === $details['status']) {
            $request->markNew();
        } else if ($details['status'] === PayboxApi::ACCEPTED_STATUS) {
            $request->markCaptured();
        } else if ($details['status'] === PayboxApi::CANCELLED_STATUS) {
            $request->markCanceled();
        } else {
            $request->markFailed();
        }
    }

    public function supports($request)
    {
        return
            $request instanceof GetStatusInterface &&
            $request->getFirstModel() instanceof PaymentInterface
        ;
    }
}
